<?php


class HealthDiagnosticModel extends BaseHealthDiagnostic
{
    private $timeCheckForBlueHost;
    private $failedHosts = [];
    private $sqlBuilders = [];

    public function __construct($userName)
    {
        parent::__construct($userName);

        $settings = json_decode(cfapi_settings_get($this->userName), JSON_OBJECT_AS_ARRAY);
        $this->timeCheckForBlueHost = time() - ($settings['data'][0]['blueHostHorizon']);
        $this->setSqlBuilders();
    }

    public function getHealthStatus()
    {
        $result = [
            self::HOSTS_NEVER_COLLECTED_TYPE => $this->getStatus($this->sqlBuilders[self::HOSTS_NEVER_COLLECTED_TYPE]),
            self::NOT_RECENTLY_COLLECTED_TYPE => $this->getStatus($this->sqlBuilders[self::NOT_RECENTLY_COLLECTED_TYPE]),
            self::SAME_IDENTITY_TYPE => $this->getStatus($this->sqlBuilders[self::SAME_IDENTITY_TYPE]),
            self::NOT_RUN_RECENTLY_TYPE => $this->getStatus($this->sqlBuilders[self::NOT_RUN_RECENTLY_TYPE]),
            self::LAST_AGENT_RUN_UNSUCCESSFUL_TYPE => $this->getStatus($this->sqlBuilders[self::LAST_AGENT_RUN_UNSUCCESSFUL_TYPE]),
            'totalFailed' => sizeof($this->failedHosts),
            'total' => $this->getTotalHostCount()
        ];

        return $result;
    }

    private function setSqlBuilders()
    {
        $this->sqlBuilders = [
            self::HOSTS_NEVER_COLLECTED_TYPE => HealthDiagnosticSqlBuilderFactory::createSqlBuilder(self::HOSTS_NEVER_COLLECTED_TYPE, $this->userName),
            self::NOT_RECENTLY_COLLECTED_TYPE => HealthDiagnosticSqlBuilderFactory::createSqlBuilder(self::NOT_RECENTLY_COLLECTED_TYPE, $this->userName)->setParams(['timeCheckForBlueHost' => $this->timeCheckForBlueHost]),
            self::SAME_IDENTITY_TYPE => HealthDiagnosticSqlBuilderFactory::createSqlBuilder(self::SAME_IDENTITY_TYPE,$this->userName),
            self::NOT_RUN_RECENTLY_TYPE => HealthDiagnosticSqlBuilderFactory::createSqlBuilder(self::NOT_RUN_RECENTLY_TYPE, $this->userName),
            self::LAST_AGENT_RUN_UNSUCCESSFUL_TYPE => HealthDiagnosticSqlBuilderFactory::createSqlBuilder(self::LAST_AGENT_RUN_UNSUCCESSFUL_TYPE, $this->userName)
        ];
    }

    public function getStatus(HealthDiagnosticSqlBuilderInterface $sqlBuilder)
    {
        try {
            $data = $this->runQuery($sqlBuilder->getSql());
            $hostKeys = [];

            foreach ($data['rows'] as $row) {
                $hostKeys[] = $row[0];
            }

            $nonOverlappedHosts = array_diff($hostKeys, $this->failedHosts);
            $this->failedHosts = array_merge($this->failedHosts, $nonOverlappedHosts);
            $count = sizeof($nonOverlappedHosts);

            return $count;
        } catch (Exception $e) {
            syslog(LOG_ERR,  $e->getTraceAsString());
            throw $e;
        }
    }

    private function getTotalHostCount()
    {
        $data = $this->runQuery('SELECT count(hostkey) FROM vm_hosts');
        return isset($data['rows'][0]) ? $data['rows'][0][0] : 0;
    }

    private function runQuery($sql)
    {
        $data = cfapi_query_post($this->userName, $sql, false, false, false, -1, [], []);
        $data = json_decode($data, JSON_OBJECT_AS_ARRAY);
        return $data['data'][0];
    }

}