<?php

/**
 * @uri /user/([^\/]+)/subscription/query/([^\/]+) 3
 */
class UserSubscriptionQuery extends CfProtectedResource
{
    function get($request, $username, $subscription_id)
    {
        $user = $this->username;

        $response = new Response($request);

        $payload = cfapi_user_subscription_query_get($user, $username, $subscription_id);
        if ($payload)
        {
            $response->code = Response::OK;
            $response->body = $payload;
        }
        else
        {
            $response->code = Response::NOTFOUND;
        }

        return $response;
    }

    function put($request, $username, $subscription_id)
    {
        $user = $this->username;

        $data = Utils::getValidJsonData($request->data);
        $output_types = $data->outputTypes;
        if (!$output_types)
        {
            $output_types = array('csv');
        }

        if ($data->hostContextInclude)
        {
            if (!is_array($data->hostContextInclude))
            {
                $response = new Response($request);
                $response->code = Response::BADREQUEST;
                $response->body = "'hostContextInclude' field must be an array";
                return $response;
            }
        }
        else
        {
            $data->hostContextInclude = array();
        }

        if ($data->hostContextExclude)
        {
            if (!is_array($data->hostContextExclude))
            {
                $response = new Response($request);
                $response->code = Response::BADREQUEST;
                $response->body = "'hostContextExclude' field must be an array";
                return $response;
            }
        }
        else
        {
            $data->hostContextExclude = array();
        }

        $response = new Response($request);

        if (cfapi_user_subscription_query_put($user,
                                              $username,
                                              $subscription_id,
                                              $data->to,
                                              $data->enabled,
                                              $data->query,
                                              $data->schedule,
                                              $data->title,
                                              $data->description,
                                              $output_types,
                                              $data->hostContextInclude,
                                              $data->hostContextExclude))
        {
            $response->code = Response::CREATED;
        }
        else
        {
            $response->code = Response::INTERNALSERVERERROR;
        }

        return $response;
    }

    function delete($request, $username, $subscription_id)
    {
        $user = $this->username;

        $response = new Response($request);

        if (cfapi_user_subscription_query_delete($user, $username, $subscription_id))
        {
            $response->code = Response::ACCEPTED;
        }
        else
        {
            $response->code = Response::INTERNALSERVERERROR;
        }

        return $response;
    }
}

/**
 * @uri /user/(.+)/subscription/query 2
 */
class UserSubscriptionQueryList extends CfProtectedResource
{
    function get($request, $username)
    {
        $user = $this->username;

        $response = new Response($request);
        $payload = cfapi_user_subscription_query_list($user, $username);
        if ($payload)
        {
            $response->code = Response::OK;
            $response->body = $payload;
        }
        else
        {
            $response->code = Response::NOTFOUND;
        }

        return $response;
    }
}

/**
 * @uri /user/(.+)/sketches/ 2
 */
class UserSketchesList extends CfProtectedResource
{
    function get($request, $username)
    {
        $user = $this->username;

        $response = new Response($request);
        $payload = cfapi_user_sketches_list($user,$username);
        if ($payload)
        {
            $response->code = Response::OK;
            $response->body = $payload;
        }
        else
        {
            $response->code = Response::NOTFOUND;
        }

        return $response;
    }
}

/**
 * @uri /user/:username 1
 */
class User extends CfProtectedResource
{
    function get($request, $username)
    {
        $user = $this->username;

        $response = new Response($request);
        $payload = cfapi_user_get($user, $username);
        if ($payload)
        {
            $response->code = Response::OK;
            $response->body = $payload;
        }
        else
        {
            $response->code = Response::NOTFOUND;
        }

        return $response;
    }

    function put($request, $username)
    {
        $user = $this->username;

        $data = Utils::getValidJsonData($request->data);

        $response = new Response($request);

        if (cfapi_user_put($user, $username, $data->password,
                $data->name, $data->email, $data->roles))
        {
            $response->code = Response::CREATED;
        }
        else
        {
            $response->code = Response::INTERNALSERVERERROR;
        }

        return $response;
    }

    function post($request, $username)
    {
        $user = $this->username;

        $data = Utils::getValidJsonData($request->data);

        $response = new Response($request);

        if (cfapi_user_post($user, $username, $data->password,
                $data->name, $data->email, $data->roles))
        {
            $response->code = Response::ACCEPTED;
        }
        else
        {
            $response->code = Response::INTERNALSERVERERROR;
        }

        return $response;
    }

    function delete($request, $username)
    {
        $user = $this->username;

        $response = new Response($request);

        if (cfapi_user_delete($user, $username))
        {
            $response->code = Response::ACCEPTED;
        }
        else
        {
            $response->code = Response::INTERNALSERVERERROR;
        }

        return $response;
    }
}

/**
 * @uri /user 0
 */
class UserList extends CfProtectedResource
{
    function get($request)
    {

        $this->accessRequired();
        $user = $this->username;

        $username_rx = Utils::queryParam('id');
        $external = Utils::checkBoolean(Utils::queryParam('external'), 'external');
        $userModel = new UsersList();

        $response = new Response($request);

        $userListRequest = new UsersListRequestStructure($user,  $username_rx,  $external ? 'external' : 'internal');
        $response->body = $userModel->getUsersList($userListRequest);
        $response->code = Response::OK;

        return $response;
    }
}
