<?php
require_once "SketchBaseTest.php";

class SketchApiTest extends SketchBaseTest
{
    public $sketchApi = null;
    public $username = 'admin';
    public $dcSettings;



    public function testGetAllSketches()
    {
        $sketches = $this->cf_sketch_api->getAllSketch(array());
        $this->assertEquals(7, count($sketches));
    }

    public function testGetAllSketchesWithEnterpriseFilter()
    {
        $filter = array(
            array(
                "tags",
                "equals",
                "enterprise_compatible"
            )
        );
        $sketches = $this->cf_sketch_api->getAllSketch($filter);
        $this->assertEquals(2, count($sketches));
        $keys = array_keys($sketches);
        $expectedArray = array(
            'Packages::installed',
            'System::set_hostname'
        );
        $this->assertEquals(asort($keys), asort($expectedArray));
    }

    public function testGetSketchInformation()
    {
        $info = $this->cf_sketch_api->getSketchInformation('Packages::installed');
        $this->assertTrue(is_array($info));
        $this->assertEquals('cfdc_packages', $info['namespace']);
    }

    public function testGetSketchActivationsNull()
    {
        $activations = $this->cf_sketch_api->getActivations('Packages::installed');
        $this->assertTrue(empty($activations));
    }

    public function testInstallOfSketch()
    {


        try
        {
            $sourcePath = $this->cf_sketch_api->getSketchSourcePath('Packages::installed');
            $result = $this->cf_sketch_api->install('Packages::installed', $sourcePath, 'sketches', true);
            $this->assertTrue(!empty($result['Packages::installed']));
        }

        catch(Exception $e)
        {
            $this->fail('should not raise exception but got exception ' . $e->getMessage());
        }
    }

    public function testDefineParams()
    {


        try
        {
            $param = array(
                'name' => 'test'
            );
            $result = $this->cf_sketch_api->defineParams('Packages::installed', 'testname', $param);
            $this->assertTrue(!empty($result['testname']));
        }

        catch(Exception $e)
        {
            $this->fail('should not raise exception but got exception ' . $e->getMessage());
        }
    }

    public function testGetDefineParams()
    {


        try
        {
            $param = array(
                'name' => 'test'
            );
            $result = $this->cf_sketch_api->defineParams('Packages::installed', 'testname', $param);
            $this->assertTrue(!empty($result['testname']));
            $definitions = $this->cf_sketch_api->getDefinitions();
            $this->assertTrue(!empty($definitions['testname']));
            $this->assertTrue(!empty($definitions['testname']['Packages::installed']));
            $this->assertEquals($param, $definitions['testname']['Packages::installed']);
        }

        catch(Exception $e)
        {
            $this->fail('should not raise exception but got exception ' . $e->getMessage());
        }
    }

    public function testEnvironmentSet()
    {


        try
        {
            $environment = array(
                "x",
                "y",
                "regex.*"
            );
            $defEnv = $this->cf_sketch_api->defineEnvironment('testing', $environment);
            $this->assertTrue(!empty($defEnv['testing']));
        }

        catch(Exception $e)
        {
            $this->fail('should not raise exception but got exception ' . $e->getMessage());
        }
    }

    public function testEnvironmentGet()
    {


        try
        {
            $environment = array(
                "x",
                "y",
                "regex.*"
            );
            $defEnv = $this->cf_sketch_api->defineEnvironment('testing', $environment);
            $this->assertTrue(!empty($defEnv['testing']));
            $getEnv = $this->cf_sketch_api->getEnvironments();
            $this->assertTrue(!empty($getEnv));
            $this->assertEquals($getEnv['testing']['activated']['include'], $environment);
        }

        catch(Exception $e)
        {
            $this->fail('should not raise exception but got exception ' . $e->getMessage());
        }
    }

    public function testActivation()
    {


        try
        {
            $sketchName = 'Packages::installed';
            $target = 'sketches';
            $envName = 'testing';
            $paramName = 'test-name';
            $environment = array(
                "cfengine_3"
            );
            $sourcePath = $this->cf_sketch_api->getSketchSourcePath('Packages::installed');
            $install = $this->cf_sketch_api->install($sketchName, $sourcePath, 'sketches', true);
            $this->assertTrue(!empty($install['Packages::installed']));
            $defEnv = $this->cf_sketch_api->defineEnvironment('testing', $environment);
            $param = array(
                'pkgs_add' => array(
                    'test'
                ) ,
                'pkgs_delete' => array(
                    'test'
                )
            );
            $result = $this->cf_sketch_api->defineParams($sketchName, $paramName, $param);
            $metaData = array(
                'identifier' => 'test-activation'
            );
            $bundle = 'installed';
            $activate = $this->cf_sketch_api->activate($sketchName, $target, $envName, $bundle, $paramName, $metaData);
        }

        catch(Exception $e)
        {
            $this->fail('should not raise exception but got exception ' . $e->getMessage());
        }
    }

    public function testActivationWithInvalidSketchName()
    {


        try
        {
            $sketchName = 'Packages::installedxxx';
            $target = 'sketches';
            $envName = 'testing';
            $paramName = 'test-name';
            $environment = array(
                "cfengine_3"
            );
            $sourcePath = $this->cf_sketch_api->getSketchSourcePath('Packages::installed');
            $install = $this->cf_sketch_api->install($sketchName, $sourcePath, 'sketches', true);
            $this->assertTrue(!empty($install['Packages::installed']));
            $defEnv = $this->cf_sketch_api->defineEnvironment('testing', $environment);
            $param = array(
                'pkgs_add' => array(
                    'test'
                ) ,
                'pkgs_delete' => array(
                    'test'
                )
            );
            $result = $this->cf_sketch_api->defineParams($sketchName, $paramName, $param);
            $metaData = array(
                'identifier' => 'test-activation'
            );
            $bundle = 'installed';
            $activate = $this->cf_sketch_api->activate($sketchName, $target, $envName, $bundle, $paramName, $metaData);
            $this->fail('should trigger exception');
        }

        catch(Exception $e)
        {
            $this->assertEquals('Sketch Packages::installedxxx could not be found in source repo' , $e->getMessage());
        }
    }

    /**
     * @depends testActivation
     */
    public function testDeActivation()
    {
        $sketchName = 'Packages::installed';
        $target = 'sketches';
        $envName = 'testing';
        $paramName = 'test-name';
        $environment = array(
            "cfengine_3"
        );
        $sourcePath = $this->cf_sketch_api->getSketchSourcePath('Packages::installed');
        $install = $this->cf_sketch_api->install($sketchName, $sourcePath, 'sketches', true);
        $this->assertTrue(!empty($install['Packages::installed']));
        $defEnv = $this->cf_sketch_api->defineEnvironment('testing', $environment);
        $param = array(
            'pkgs_add' => array(
                'test'
            ) ,
            'pkgs_delete' => array(
                'test'
            )
        );
        $result = $this->cf_sketch_api->defineParams($sketchName, $paramName, $param);
        $metaData = array(
            'identifier' => 'test-activation'
        );
        $bundle = 'installed';
        $activate = $this->cf_sketch_api->activate($sketchName, $target, $envName, $bundle, $paramName, $metaData);
        $apiResponse = $this->cf_sketch_api->deactivate('test-activation', $sketchName, $target);
        $this->assertTrue(!empty($apiResponse));
    }
    /**
     * @depends testActivation
     */
    public function testShouldRemoveDefinedVariablesAfterDeactivation()
    {
        $sketchName = 'Packages::installed';
        $target = 'sketches';
        $envName = 'testing';
        $paramName = 'test-name';
        $environment = array(
            "cfengine_3"
        );
        $sourcePath = $this->cf_sketch_api->getSketchSourcePath('Packages::installed');
        $install = $this->cf_sketch_api->install($sketchName, $sourcePath, 'sketches', true);
        $this->assertTrue(!empty($install['Packages::installed']));
        $defEnv = $this->cf_sketch_api->defineEnvironment('testing', $environment);
        $param = array(
            'pkgs_add' => array(
                'test'
            ) ,
            'pkgs_delete' => array(
                'test'
            )
        );
        $result = $this->cf_sketch_api->defineParams($sketchName, $paramName, $param);
        $metaData = array(
            'identifier' => 'test-activation'
        );
        $bundle = 'installed';
        $activate = $this->cf_sketch_api->activate($sketchName, $target, $envName, $bundle, $paramName, $metaData);
        $apiResponse = $this->cf_sketch_api->deactivate('test-activation', $sketchName, $target);
        // re query definition
        $definitions = $this->cf_sketch_api->getDefinitions();
        $this->assertFalse(array_key_exists('test-name', $definitions) , "should not have the param defined");
    }

    public function testEmptyGetActivationDetail()
    {
        $emptyResult = $this->cf_sketch_api->get_activation_details('fake','fake');
        $this->assertTrue(empty($emptyResult));
    }

    public function testGetActivationDetail()
    {
        $sketchName = 'Packages::installed';
        $target = 'sketches';
        $envName = 'testing';
        $paramName = 'test-name';
        $environment = array(
            "cfengine_3"
        );
        $sourcePath = $this->cf_sketch_api->getSketchSourcePath($sketchName);
        $install = $this->cf_sketch_api->install($sketchName, $sourcePath, 'sketches', true);
        $this->assertTrue(!empty($install['Packages::installed']));
        $defEnv = $this->cf_sketch_api->defineEnvironment('testing', $environment);
        $param = array(
            'pkgs_add' => array(
                'test'
            ) ,
            'pkgs_delete' => array(
                'test'
            )
        );
        $result = $this->cf_sketch_api->defineParams($sketchName, $paramName, $param);
        $metaData = array(
            'identifier' => 'test-activation'
        );
        $bundle = 'installed';
        $activate = $this->cf_sketch_api->activate($sketchName, $target, $envName, $bundle, $paramName, $metaData);
        $accDetails = $this->cf_sketch_api->get_activation_details('test-activation',$sketchName);
        $this->assertTrue(!empty($accDetails));
        $this->assertEquals($accDetails['params'][0], $paramName);
        $this->assertEquals($accDetails['environment'], $envName);
    }

    public function testCheckEnvironmentInOtherActivation()
    {
        $sketchName = 'Packages::installed';
        $target = 'sketches';
        $envName = 'testing';
        $paramName = 'test-name';
        $environment = array(
            "cfengine_3"
        );
        $sourcePath = $this->cf_sketch_api->getSketchSourcePath($sketchName);
        $install = $this->cf_sketch_api->install($sketchName, $sourcePath, 'sketches', true);
        $this->assertTrue(!empty($install['Packages::installed']));
        $defEnv = $this->cf_sketch_api->defineEnvironment('testing', $environment);
        $param = array(
            'pkgs_add' => array(
                'test'
            ) ,
            'pkgs_delete' => array(
                'test'
            )
        );
        $result = $this->cf_sketch_api->defineParams($sketchName, $paramName, $param);
        $metaData = array(
            'identifier' => 'test-activation'
        );
        $bundle = 'installed';
        $activate = $this->cf_sketch_api->activate($sketchName, $target, $envName, $bundle, $paramName, $metaData);
        $accDetails = $this->cf_sketch_api->get_activation_details('test-activation',$sketchName);
        $this->assertFalse($this->cf_sketch_api->checkActivationEnvironment('test-activation', 'testing'));
    }

    public function testCheckEnvironmentInOtherActivationWhenUsed()
    {
        $sketchName = 'Packages::installed';
        $target = 'sketches';
        $envName = 'testing';
        $paramName = 'test-name';
        $environment = array(
            "cfengine_3"
        );
        $sourcePath = $this->cf_sketch_api->getSketchSourcePath($sketchName);
        $install = $this->cf_sketch_api->install($sketchName, $sourcePath, 'sketches', true);
        $this->assertTrue(!empty($install['Packages::installed']));
        $defEnv = $this->cf_sketch_api->defineEnvironment('testing', $environment);
        $param = array(
            'pkgs_add' => array(
                'test'
            ) ,
            'pkgs_delete' => array(
                'test'
            )
        );
        $result = $this->cf_sketch_api->defineParams($sketchName, $paramName, $param);
        $metaData = array(
            'identifier' => 'test-activation'
        );
        $bundle = 'installed';
        $activate = $this->cf_sketch_api->activate($sketchName, $target, $envName, $bundle, $paramName, $metaData);
        $metaData2 = array(
            'identifier' => 'test-activation2'
        );
        $activate = $this->cf_sketch_api->activate($sketchName, $target, $envName, $bundle, $paramName, $metaData2);
        $accDetails = $this->cf_sketch_api->get_activation_details('test-activation',$sketchName);
        $this->assertTrue($this->cf_sketch_api->checkActivationEnvironment('test-activation', 'testing'));
    }

    public function testValidation()
    {
        $validation = $this->cf_sketch_api->getValidations();
        $this->assertEquals($validation['DIGITS'], array(
            'valid_regex' => '^[0-9]+$'
        ));
    }

    public function testValidationArray()
    {


        try
        {
            $result = $this->cf_sketch_api->validate('AWS_S3_VALIDATION', array(
                'xx' => 'xx'
            ));
            $this->fail("Should generate exception");
        }

        catch(Exception $e)
        {
            $message = $e->getMessage();
            $this->assertEquals($message, "The key 'xx' is not valid.");
        }
    }

    public function tearDown()
    {
        $dirPath = '/tmp/dctest';


        foreach (new RecursiveIteratorIterator(new RecursiveDirectoryIterator($dirPath, FilesystemIterator::SKIP_DOTS) , RecursiveIteratorIterator::CHILD_FIRST) as $path)
        {
            $path->isFile() ? unlink($path->getPathname()) : rmdir($path->getPathname());
        }

        rmdir($dirPath);

        parent::tearDown();
    }
}
