<?php
require_once "SketchBaseTest.php";

class SketchRestTest extends SketchBaseTest
{

  public function testListAllSketches()
  {

    try
    {
      $sketches = $this->getResults('/dc/sketch');
      $this->assertEquals(200, $this->pest->lastStatus());
      $this->assertValidJson($sketches);
      $this->assertEquals(7, count($sketches));
    }
    catch(Pest_Exception $e)
    {
      $this->fail($e);
    }
  }

  public function testListSketchesWithNameParameter()
  {

    try
    {
      $name = 'Webserver::Install';
      $sketches = $this->getResults('/dc/sketch?name=' . $name);
      $this->assertEquals(200, $this->pest->lastStatus());
      $this->assertValidJson($sketches);
      $this->assertEquals(1, count($sketches));
      $this->assertTrue(array_key_exists($name, $sketches));
    }
    catch(Pest_Exception $e)
    {
      $this->fail($e);
    }
  }

  public function testSketchesWithName()
  {

    try
    {
      $name = 'Webserver::Install';
      $sketches = $this->getResults('/dc/sketch/' . $name);
      $this->assertEquals(200, $this->pest->lastStatus());
      $this->assertValidJson($sketches);
      $this->assertEquals($name, $sketches[0]['metadata']['name']);
    }
    catch(Pest_Exception $e)
    {
      $this->fail($e);
    }
  }

  public function testSketchesEmptyActivationList()
  {

    try
    {
      $data = $this->getResults('/dc/activation/');
      $this->assertEquals(200, $this->pest->lastStatus());
      $this->assertValidJson($data);
      $this->assertEmpty($data);
    }
    catch(Pest_Exception $e)
    {
      $this->fail($e);
    }
  }

  public function testSketchInstall()
  {

    try
    {
      $name = 'System::set_hostname';
      $result = $this->pest->put('/dc/sketch/' . $name, null);
      $this->assertEquals(200, $this->pest->lastStatus());
      $this->assertTrue(array_key_exists($name, $result));
    }
    catch(Pest_Exception $e)
    {
      $this->fail($e);
    }
  }

  public function testSketchInstallWithInvalidName()
  {

    try
    {
      $name = 'System::set_hostnamexxx';
      $result = $this->pest->put('/dc/sketch/' . $name, null);
      $this->fail('should trigger exception');
    }
    catch(Pest_Exception $e)
    {
      $this->assertEquals(500, $this->pest->lastStatus());
    }
  }

  public function testSketchDefineParameter()
  {
    try
    {
      $paramname = 'set_hostname_param';
      $sketchName = 'System::set_hostname';

      $param = array(
        'sketchName' => $sketchName,
        'params' => array(
          'hostname' => 'test',
          'domainname' => 'test'
        )
      );

      $result = $this->pest->put('/dc/definition/' . $paramname, json_encode($param));
      $this->assertEquals(200, $this->pest->lastStatus());
      $data  = $this->getResults('/dc/definition');
      $this->assertEquals('test',$data[$paramname][$sketchName]['hostname']);
      $this->assertEquals('test',$data[$paramname][$sketchName]['domainname']);

    }
    catch(Pest_Exception $e)
    {
      $this->fail($e);
    }
  }

  public function testSketchDefineParameterWithIllegalCharacters()
  {
    try
    {
      $paramname = 'set_hostname_param22&%@';
      $sketchName = 'System::set_hostname';

      $param = array(
        'sketchName' => $sketchName,
        'params' => array(
          'hostname' => 'test',
          'domainname' => 'test'
        )
      );

      $result = $this->pest->put('/dc/definition/' . $paramname, json_encode($param));
      $this->fail('should throw exception');

    }
    catch(Pest_Exception $e)
    {
      $this->assertEquals(500, $this->pest->lastStatus());
      $this->assertEquals('Parameter name can only contain alpha numeric with exception of - or _  characters',$this->pest->lastBody());
    }
  }

  public function testSketchReDefineParameter()
  {
    try
    {

      $this->testSketchDefineParameter();
      $paramname = 'set_hostname_param';
      $sketchName = 'System::set_hostname';
      $param = array(
        'sketchName' => $sketchName,
        'params' => array(
          'hostname' => 'testRedefine',
          'domainname' => 'testRedefine'
        )
      );

      $result = $this->pest->put('/dc/definition/' . $paramname, json_encode($param));
      $this->assertEquals(200, $this->pest->lastStatus());


      $data  = $this->getResults('/dc/definition');
      $this->assertEquals('testRedefine',$data[$paramname][$sketchName]['hostname']);
      $this->assertEquals('testRedefine',$data[$paramname][$sketchName]['domainname']);
    }
    catch(Pest_Exception $e)
    {
      $this->fail($e);
    }
  }

  public function testSketchDefineParameterWithSketchNameMising()
  {
    try
    {
      $paramname = 'set_hostname_param';
      $param = array(
        'params' => array(
          'hostname' => 'test',
          'domainname' => 'test'
        )
      );

      $result = $this->pest->put('/dc/definition/' . $paramname, json_encode($param));
      $this->fail($e);

    }
    catch(Pest_Exception $e)
    {
      $this->assertEquals(400, $this->pest->lastStatus());
      $message = $this->pest->lastBody();
      $this->assertEquals('sketchName is a required parameter', $message);

    }
  }

  public function testSketchDefineParameterWithInvalidSketchName()
  {
    try
    {
      $paramname = 'set_hostname_param';
      $param = array(
        'sketchName' => 'System::set_hostnamexxx',
        'params' => array(
          'hostname' => 'test',
          'domainname' => 'test'
        )
      );

      $result = $this->pest->put('/dc/definition/' . $paramname, json_encode($param));
      $this->fail('should trigger exception');
    }
    catch(Pest_Exception $e)
    {
      $this->assertEquals(500, $this->pest->lastStatus());
      $this->assertEquals('Sketch named System::set_hostnamexxx not found.',$e->getMessage());
    }
  }

  public function testSketchDefineEnvironment()
  {
    try
    {
      $envname = 'set_hostname_env';
      $param = array(
        'environment' => array(
          "cfengine_3"
        )
      );

      $result = $this->pest->put('/dc/environment/' . $envname, json_encode($param));
      $this->assertEquals(200, $this->pest->lastStatus());

      $data = $this->getResults('/dc/environment/');
      $this->assertEquals(200, $this->pest->lastStatus());
      $this->assertTrue(array_key_exists($envname, $data));
      $this->assertEquals('cfengine_3',$data[$envname]['activated']['include'][0]);
    }
    catch(Pest_Exception $e)
    {
      $this->fail($e);
    }
  }

  public function testSketchDefineEnvironmentWithMissingRequiredParams()
  {
    try
    {
      $envname = 'set_hostname_env';
      $param = array(

      );

      $result = $this->pest->put('/dc/environment/' . $envname, json_encode($param));
      $this->assertEquals(200, $this->pest->lastStatus());

      $data = $this->getResults('/dc/environment/');
      $this->fail('Should trigger exception');

    }
    catch(Pest_Exception $e)
    {
        $this->assertEquals(400, $this->pest->lastStatus());
        $this->assertEquals('environment is a required parameter', $this->pest->lastBody());
    }
  }


  public function testSketchDefineEnvironmentWithInvalidName()
  {
    try
    {
      $envname = 'set_hostname_env%276@';
      $param = array(
        'environment' => array(
          "cfengine_3"
        )
      );

      $result = $this->pest->put('/dc/environment/' . $envname, json_encode($param));
      $this->fail('Should trigger exception');

    }
    catch(Pest_Exception $e)
    {
        $this->assertEquals(500, $this->pest->lastStatus());
        $this->assertEquals('Environment name can only contain alpha numeric with exception of - or _  characters', $this->pest->lastBody());
    }
  }

  public function testSketchReDefineEnvironment()
  {
    try
    {

      $this->testSketchDefineEnvironment();


      $envname = 'set_hostname_env';
      $param = array(
        'environment' => array(
          "windows"
        )
      );

      $result = $this->pest->put('/dc/environment/' . $envname, json_encode($param));
      $this->assertEquals(200, $this->pest->lastStatus());

      $data = $this->getResults('/dc/environment/');
      $this->assertEquals(200, $this->pest->lastStatus());
      $this->assertTrue(array_key_exists($envname, $data));
      $this->assertEquals('windows',$data[$envname]['activated']['include'][0]);
    }
    catch(Pest_Exception $e)
    {
      $this->fail($e);
    }
  }


  public function testActivation()
  {

    try
    {
      $name = 'test%20-123_xx';
      $this->testSketchDefineEnvironment();
      $this->testSketchDefineParameter();

      $param = array(
        'environmentName' => 'set_hostname_env',
        'paramName' => 'set_hostname_param',
        'bundleName' => 'set'
      );
      $data = $this->pest->put('/dc/activation/'.$name.'/System::set_hostname', json_encode($param));
      $this->assertEquals(200, $this->pest->lastStatus());

      $activationDetail = $this->getResults('/dc/activation/'.$name.'/System::set_hostname?details=1');
      $this->assertEquals('cfengine_3',$activationDetail[0]['details']['environments']['activated']['include'][0]);


    }
    catch(Pest_Exception $e)
    {
      $this->fail($e);
    }
  }

  public function testActivationWithMissingRequiredParams()
  {

    try
    {
      $this->testSketchDefineEnvironment();
      $this->testSketchDefineParameter();

      $param = array(
        'environmentName' => 'set_hostname_env'
      );
      $data = $this->pest->put('/dc/activation/test/System::set_hostname', json_encode($param));
      $this->fail('should raise exception');
    }
    catch(Pest_Exception $e)
    {
      $this->assertEquals(400, $this->pest->lastStatus());
      $this->assertEquals('bundleName is a required parameter', $this->pest->lastBody());
    }
  }


  public function testActivationWithInvalidName()
  {

    try
    {
      $this->testSketchDefineEnvironment();
      $this->testSketchDefineParameter();
      $actName = 'aasdad.dsasad!';

      $param = array(
        'environmentName' => 'set_hostname_env',
        'paramName' => 'set_hostname_param',
        'bundleName' => 'set'
      );
      $data = $this->pest->put('/dc/activation/'.$actName.'/System::set_hostname', json_encode($param));
      $this->fail('should raise exception');
    }
    catch(Pest_Exception $e)
    {
      $this->assertEquals(500, $this->pest->lastStatus());
      $this->assertEquals('Sketch activation name can only contain alpha numeric with exception of spaces - or _  characters', $this->pest->lastBody());
    }
  }






  public function testActivationWithDifferentEnvironment()
  {

    try
    {

      $this->testActivation();
      $this->testSketchReDefineEnvironment();

      $param = array(
        'environmentName' => 'set_hostname_env',
        'paramName' => 'set_hostname_param',
        'bundleName' => 'set'
      );
      $data = $this->pest->put('/dc/activation/test2/System::set_hostname', json_encode($param));
      $this->assertEquals(200, $this->pest->lastStatus());

      $activationDetail = $this->getResults('/dc/activation/test2/System::set_hostname?details=1');
      $this->assertEquals('windows',$activationDetail[0]['details']['environments']['activated']['include'][0]);


    }
    catch(Pest_Exception $e)
    {
      $this->fail($e);
    }
  }




  public function testReActivateWithSameActivationName()
  {

    try
    {
      $name = 'test%20-123_xx';
      $this->testActivation();
      $param = array(
        'environmentName' => 'set_hostname_env',
        'paramName' => 'set_hostname_param',
        'bundleName' => 'set'
      );

      $data = $this->pest->put('/dc/activation/'.$name.'/System::set_hostname', json_encode($param));
      $this->fail('should throw exception');
    }
    catch(Pest_Exception $e)
    {
      $this->assertEquals(400, $this->pest->lastStatus());
      $this->assertEquals('Sketch activation with same name already exists.', $this->pest->lastBody());


    }
  }





  public function testGetValidationList()
  {

    $data = $this->getResults('/dc/validation/');
    $this->assertEquals(200, $this->pest->lastStatus());
    $this->assertValidJson($data);
    $this->assertEquals(22, count($data));
  }

  public function testGetValidationListWithFilter()
  {

    $data = $this->getResults('/dc/validation/?filter=DIGITS,HOSTNAME');
    $this->assertEquals(200, $this->pest->lastStatus());
    $this->assertValidJson($data);
    $this->assertEquals(2, count($data));
  }

  public function testActivationWithInvalidJson()
  {

    try
    {
      $data = $this->pest->put('/dc/activation/test/System::set_hostname', '{
                  invalid json
                }');
      $this->fail('Should trigger exception');
    }
    catch(Pest_Exception $e)
    {
      $this->assertEquals('Unable to parse JSON payload', $e->getMessage());
      $this->assertEquals(400, $this->pest->lastStatus());
    }
  }
}
