<?php
require_once APPPATH . 'modules/dashboard/models/entities/CF_Dashboard.php';
require_once APPPATH . 'modules/dashboard/models/entities/CF_Alert.php';

class dashboard_model extends Cf_Model
{

    var $collectionName = 'dashboard_dashboards';
    var $userCollectionName = 'users';

    var $defaultFilter;

    function __construct()
    {
        parent::__construct();
    }

    function setDefaultFilter($filter)
    {
        $this->defaultFilter = $filter;
    }

    /*
     * Get all dashboards
     */
    function get_all_dashboards($filter = array())
    {

      $query = $this->db->where($filter)->get($this->collectionName);
      $result = array();


      foreach ((array)$query->result_array() as $dashResult)
      {
          $obj = new CF_Dashboard($dashResult);
          $result[] = $obj;
      }

      return $result;

    }


    /*
     * Get all public dashboards - filter is 'public' boolean == true
     */
    function get_public_dashboards($filter = array())
    {
        $query = $this->db->where($filter)->get($this->collectionName);
        $result = array();


        foreach ((array)$query->result_array() as $dashResult)
        {
            $obj = new CF_Dashboard($dashResult);
            $result[] = $obj;
        }

        return $result;
    }

    /*
     * Get all users's dashboards - filter is username only
     */
    function get_user_allowed_dashboards($filter = array())
    {
        if (!isset($filter['id'])) 
        {
          $query = $this->db->where('public', '1')->or_where('username', $filter['username'])->get($this->collectionName);
        }
        else // narrow the result by id if it is set
        {
          $query = $this->db->where('id', $filter['id'])->where('public', '1')->or_where('username', $filter['username'])->get($this->collectionName);
        }
        
        $result = array();


        foreach ((array)$query->result_array() as $dashResult)
        {
            $obj = new CF_Dashboard($dashResult);
            $result[] = $obj;
        }

        return $result;
    }

    function get_user_dashboards($filter = array())
    {
        if (isset($filter['username']))
        {
          $query = $this->db->where('username', $filter['username'])->get($this->collectionName);
        }
        
        $result = array();

        foreach ((array)$query->result_array() as $dashResult)
        {
            $obj = new CF_Dashboard($dashResult);
            $result[] = $obj;
        }

        return $result;
    }

    function reset_user_selection($filter = '')
    {
        if (!is_array($filter))
        {
            $this->setError('Invalid parameter passed to function');
            return false;
        }

        $data = array('dashboard'=>NULL);
        $this->db->where($filter);
           
        try
        {
             $this->db->update($this->userCollectionName, $data);
        }
        catch(Exception $e)
        {
            log_message('error', $e->getMessage() . " " . $e->getFile() . " line:" . $e->getLine());
            throw $e;
        }

        return;
    }

    function remove_dashboard($filter = '')
    {

        if (!is_array($filter))
        {
            $this->setError('Invalid parameter passed to function');
            return false;
        }

        try
        {
            $return = $this->db->where($filter)->delete($this->collectionName);
            return $return;
        }
        catch(Exception $e)
        {
            log_message('error', $e->getMessage() . " " . $e->getFile() . " line:" . $e->getLine());
            throw $e;
        }
    }

    function save_dashboard($data, $overwrite)
    {

      if (!is_array($data))
      {
          $this->setError('Invalid parameter passed to function');
          return false;
      }

      $id = '';

      $filter['username'] = $data['username'];

      if (isset($data['id']))
      {
          $filter['id'] = intval($data['id']);
          unset($data['id']);
      }
      else
      {
           $filter['name']     = $data['name'];
      }

      if ($this->validate($data, $overwrite) === FALSE)
      {
          $message = $this->getErrors();
          $text = $this->lang->line('dashboard_unable_to_save'). ' ' . $message[0]['text'];
          throw new Exception($text);
      }

      $currentItem = $this->get_all_dashboards($filter);

      if (!empty($currentItem))
      {
          $id = $currentItem[0]->id;
          $owner = $currentItem[0]->getUsername();

          // $data['username'] - user who created wizard, in most cases current logged user
          if (($owner !== $data['username']))
          {
              $data['username'] = $owner;
          }
      }

      if ($id == '')
      {
          // not exist - insert it will return inserted item
          $this->db->insert($this->collectionName, $data);
          $id =  $this->db->insert_id();
      }
      else
      {
          // if exist - update, and then select inserted item
          // TURNS ARRAY INTO COMMA STRING , MIGHT NEED TO FIX when real DB
          if(is_array($data['widgets']))
          {
              $comma_separated = implode(",", $data['widgets']);
              $data['widgets'] = $comma_separated;
          }
          $this->db->where(array('id' => $id))->update($this->collectionName, $data);
      }

      // get object from DB
      $offset = '';
      $limit = '';
      $order = array();

      $objTmp = $this->get_all_dashboards(array('id' => $id));

      if ($objTmp)
      {
          $dashboard = $objTmp[0];
          return $dashboard;
      }
      else
      { // shouldn't happen
          throw new Exception($this->lang->line('dashboard_widget_unable_to_load'));
      }
    }

    /*
     * Reorder widgets and updates dahboard where ID and USERNAME match current user and selected ID
     */
    function reorder_widgets($filter = array(), $data = array())
    {
        if (!is_array($filter))
        {
            $this->setError('Invalid parameter passed to function');
            return false;
        }
        if (!is_array($data))
        {
            $this->setError('Invalid parameter passed to function');
            return false;
        }

        // if exist - update, and then select inserted item
        // TURNS ARRAY INTO COMMA STRING , MIGHT NEED TO FIX when real DB
        $comma_separated = implode(",", $data['widgets']);
        $data = $comma_separated;

        $this->db->where($filter)->update($this->collectionName, array('widgets' => $data));
        $objTmp = $this->get_all_dashboards($filter);

        if ($objTmp)
        {
            $dashboard = $objTmp[0];
            return $dashboard;
        }
        else
        { // shouldn't happen
            throw new Exception($this->lang->line('dashboard_widget_unable_to_load'));
        }
    }

    /*
     * save selected dashboard to user's data
     */
    function set_user_selection($filter = array())
    {

        if (!is_array($filter))
        {
            $this->setError('Invalid parameter passed to function');
            return false;
        }

        $this->db->where(array('username' => $filter['username']));
        $query = $this->db->get($this->userCollectionName);
        $userObj = $query->result_array();
        

        if ($userObj)
        {
            $userObj[0]['dashboard'] = $filter['id'];
            $this->db->where(array('username' => $filter['username']))->update($this->userCollectionName, $userObj[0]);
            return $userObj[0];
        }

        return;
    }

     /*
     * get selected dashboard for user
     */
    function get_user_selection($filter = array())
    {

        $this->db->select('dashboard, username');
        $this->db->where($filter);
        $query = $this->db->get($this->userCollectionName);
        $result = $query->result_array();

        return $result;
    }

    function add_widget($data = array())
    {
        if (!is_array($data))
        {
            $this->setError('Invalid parameter passed to function');
            return false;
        }
        $username = $data['username'];
        $dashId = $this->get_user_selection(array('username' => $username));
        $id = $dashId[0]['dashboard'];
        if ($id)
        {
            $objTmp = $this->get_all_dashboards(array('id' => $id));
            
            if ($objTmp)
            {
                $dashboard = $objTmp[0]; 
                $widgets = $dashboard->getWidgets();
                // check for duplicates
                if(!in_array($data['id'], $widgets, true))
                {
                    array_push($widgets, $data['id']);
                }
               
                $dashboard->setWidgets($widgets);
                $dasharray = (array) $dashboard;

                // TURNS ARRAY INTO COMMA STRING , MIGHT NEED TO FIX
                $comma_separated = implode(",", $dasharray['widgets']);
                
                $dasharray['widgets'] = $comma_separated;
                $savedDash = $this->save_dashboard($dasharray, TRUE);
                return $savedDash;
            }
            else
            { // shouldn't happen
                throw new Exception($this->lang->line('dashboard_widget_unable_to_load'));
            }
        }
        else
        {
          throw new Exception($this->lang->line('dashboard_widget_unable_to_load'));
        }

    }

    function delete_widget($data = array())
    {
        if (!is_array($data))
        {
            $this->setError('Invalid parameter passed to function');
            return false;
        }

       
        $dashId = $this->get_user_selection(array('username' => $data['username']));
        $dashboards = $this->get_all_dashboards(array('id' => $dashId[0]['dashboard']));
              
        foreach ($dashboards as $dashboard)
        {
            $widgets = $dashboard->getWidgets();
            if(($key = array_search($data['id'], $widgets)) !== false) {
                unset($widgets[$key]);
                $dashboard->setWidgets($widgets);

                $dasharray = (array) $dashboard;

                // TURNS ARRAY INTO COMMA STRING , MIGHT NEED TO FIX
                $comma_separated = implode(",", $dasharray['widgets']);
                
                $dasharray['widgets'] = $comma_separated;
                $savedDash = $this->save_dashboard($dasharray, TRUE);
            }
        }

        return;

    }

    /*
     * validate the dashboard data before saving
     */
    function validate($data, $overwrite = FALSE, $is_admin = FALSE)
    {
        $valid = true;

        if (empty($data['username']) || trim($data['username']) == '')
        {
            $valid = false;
            $this->setError($this->lang->line('dashboard_empty_user_name'));
            return false;
        }

        if (empty($data['name']) || trim($data['name']) == '')
        {
            $valid = false;
            $this->setError($this->lang->line('dashboard_empty_name'));
            return false;
        }


        //check if not an overwrite & if name already taken
        if ($overwrite === false)
        {
            $item = $this->_nameAlreadyExist($data);

            if(!empty($item)) {
                $valid = false;
                $this->setError($this->lang->line('dashboard_already_exists'), 422); //422 Unprocessable Entity
            }
        }

        //check if an overwrite, then if user is not owner of this dashboard - do not allow to save
        if ($this->_nameAlreadyExist($data) === TRUE && ($this->_checkOwner($data) === FALSE) )
        {
            $this->clearErrors();
            $this->setError($this->lang->line('dashboard_already_exists'), 422); //422 Unprocessable Entity
            $valid = false;
        }

        return $valid;
    }

    /*
     * check that there is not already a dashboard with that name in DB
     */
    function _nameAlreadyExist($data)
    {
        $alreadyExist = FALSE;
        $filter = $this->defaultFilter;
        $filter['name'] = $data['name'];

        $result = $this->get_all_dashboards($filter);

        if (!empty($result))
        {
            $alreadyExist = TRUE;
        }

        return $alreadyExist;
    }

    function _checkOwner($data)
    {
        $isOwner = false;
        $filter = $this->defaultFilter;
        $filter['name'] = $data['name'];

        $currentItem = $this->get_all_dashboards($filter);

        if (!empty($currentItem))
        {
            $owner = $currentItem[0]->getUsername();

            if ($owner === $data['username'])
            {
                $isOwner = true;
            }
        }

        return $isOwner;
    }

     //************************************
    //
    // TODO: This functions should be revieed and replaced in all project if valuable
    // right now this is a copy/paste from save_search model
    //________________________________________________________

    function setError($errorText, $errorCode = 406, $level = 'ERROR')
    {
        $i = count($this->errors);
        $this->errors[$i]['text']      = $errorText;
        $this->errors[$i]['errorCode'] = $errorCode;
    }

    function getErrors($level = 'ERROR')
    {
        return $this->errors;
    }

    function clearErrors()
    {
        $this->errors = array();
    }

    // END
    // ________________________________________________________


}

?>