'use strict';

var reports = angular.module('reports', [
        'ngSanitize',
        'ui',
        'ngRoute',
        'reportServiceModule',
        'inventoryServiceModule',
        'changesServiceModule',
        'commonDirectives',
        'env.environmentServiceModule',
        'dc.naviTreeService',
        'ngCookies',
        'ui.bootstrap',
        'ngAnimate',
        'highcharts-ng',
        'inventoryDirectives',
        'inclExclDirectives',
        'inclExcl.naviTreeService',
        'includeExcludeServiceModule',
        'inclExcl.paginationServiceModule',
        'inventoryHelpers',
        'changesDirectives',
        'changesHelpers',
        'daterangepicker',
        'inventoryDataServiceModule',
        'inventoryFilterServiceModule',
        'healthDiagnosticsServiceModule'
    ],
 function($routeProvider, $locationProvider) {

            //reports created or subscribed by user
            $routeProvider.when('/index', {
                templateUrl: MP_SITE_URL + '/advancedreports/partials/listreports',
                controller: indexController
            });

            // all available reports (only reports allowed by permissins) without own reports
            $routeProvider.when('/allreports', {
                templateUrl: MP_SITE_URL + '/advancedreports/partials/allreports',
                controller: allreportsController
            });


            /*
             * Run report without param, this means that SQL string saved inside scope
             */
            $routeProvider.when('/report/run', {
                templateUrl: MP_SITE_URL + '/advancedreports/partials/reportresults',
                controller: resultController
            });

            /*
             * Run specific report
             */
            $routeProvider.when('/report/run/:reportid/', {
                templateUrl: MP_SITE_URL + '/advancedreports/partials/reportresults',
                controller: resultController
            });

            $routeProvider.when('/report/external', {
                templateUrl: MP_SITE_URL + '/advancedreports/partials/reportresults',
                controller: resultController
            });

            $routeProvider.when('/report/external/sql/:sql/', {
                templateUrl: MP_SITE_URL + '/advancedreports/partials/reportresults',
                controller: resultController
            });

            /*
             *  Open empty SQLWIZARD page
             */
            $routeProvider.when('/report/new', {
                templateUrl: MP_SITE_URL + '/advancedreports/partials/editreport',
                controller: editController
            });

            // when we run new report and then we want to edit it
            $routeProvider.when('/report/edit', {
                templateUrl: MP_SITE_URL + '/advancedreports/partials/editreport',
                controller: editController
            });
            /*
             * Edit report  by id
             */
            $routeProvider.when('/report/edit/:reportid/', {
                templateUrl: MP_SITE_URL + '/advancedreports/partials/editreport',
                controller: editController
            });


            $routeProvider.when('/report/schedule', {
                templateUrl: MP_SITE_URL + '/advancedreports/partials/schedulereport',
                controller: scheduleController
            });

            $routeProvider.when('/report/schedule/:reportid/', {
                templateUrl: MP_SITE_URL + '/advancedreports/partials/schedulereport',
                controller: scheduleController
            });


            $routeProvider.when('/inventory', {
                templateUrl: MP_SITE_URL + '/advancedreports/partials/inventory',
                controller: inventoryController
            });

            $routeProvider.when('/changes', {
                templateUrl: MP_SITE_URL + '/advancedreports/partials/changes',
                controller: changesController
            });

            $routeProvider.when('/changes/run/:reportid', {
                templateUrl: MP_SITE_URL + '/advancedreports/partials/changes',
                controller: changesController
            });

            $routeProvider.when('/inventory/run/:reportid', {
                templateUrl: MP_SITE_URL + '/advancedreports/partials/inventory',
                controller: inventoryController
            });

            $routeProvider.when('/inventory/alert/:alertid', {
                templateUrl: MP_SITE_URL + '/advancedreports/partials/inventory',
                controller: inventoryController
            });

            /*
             * Health Diagnostic reports routes
             */
            $routeProvider.when('/health-diagnostic/not-recently-collected', {
                templateUrl: MP_SITE_URL + '/advancedreports/hostDiagnosticView/notRecentlyCollected',
                controller: healthDiagnosticController
            });

            $routeProvider.when('/health-diagnostic/last-agent-run-unsuccessful', {
                templateUrl: MP_SITE_URL + '/advancedreports/hostDiagnosticView/lastAgentRunUnsuccessful',
                controller: healthDiagnosticController
            });

            $routeProvider.when('/health-diagnostic/hosts-using-same-identity', {
                templateUrl: MP_SITE_URL + '/advancedreports/hostDiagnosticView/hostsUsingSameIdentity',
                controller: healthDiagnosticController
            });

           $routeProvider.when('/health-diagnostic/hosts-never-collected', {
                templateUrl: MP_SITE_URL + '/advancedreports/hostDiagnosticView/hostsNeverCollected',
                controller: healthDiagnosticController
            });

           $routeProvider.when('/health-diagnostic/agent-not-run-recently', {
                templateUrl: MP_SITE_URL + '/advancedreports/hostDiagnosticView/agentNotRunRecently',
                controller: healthDiagnosticController
            });



            /*
             * Default  - show reports list
             */
            $routeProvider.otherwise({redirectTo: '/index'});

            // configure html5 to get links working
            // If you don't do this, you URLs will be base.com/#/home rather than base.com/home
            //$locationProvider.html5Mode(true);
        }
);

reports.factory('authHttpResponseInterceptor', ['$q', '$location', function ($q) {
    return {
        response: function (response) {
            if (response.status === 401) {
                unAuthorizedHandler();
            }
            return response || $q.when(response);
        },
        responseError: function (rejection) {
            if (rejection.status === 401) {
                unAuthorizedHandler();
            }
            return $q.reject(rejection);
        }
    }
}]);

reports.config(['$httpProvider', '$provide', function($httpProvider, $provide) {
        //This header was removed from recent version of angular.
        // PHP looks at this header to separate request between regular and AJAX.
        $httpProvider.defaults.headers.common["X-Requested-With"] = 'XMLHttpRequest';
        // IE fix to prevent caching
        $httpProvider.defaults.headers.common["If-Modified-Since"] = 0;
        // add interceptor to httpProvider
        $httpProvider.interceptors.push('authHttpResponseInterceptor');


        $provide.decorator('$exceptionHandler',
        ['$delegate', errorLogger]);
}]);


function errorLogger($delegate) {
    return function (exception, cause) {
        $delegate(exception, cause);
        var msg = exception.message + "\n" + exception.stack;
        globalLogger.sendErrorToServer(msg)
    };
}



reports.directive('testforlast', function() {
    return function(scope, element, attrs) {
        scope.$watch('$last', function(v) {
            if (v) {
                addUserLabelClass();
            }
        });
    };
});