/*
 * CFEngine Mission Common Utility functions
 */
var common = (function(){


                var _spinnerCounter = 0;
                var showSpinner =  function() {
                    _spinnerCounter++;
                    $("#globalSpinner").show();
                };
                var hideSpinner= function(flag) {
                    _spinnerCounter--;
                    if (_spinnerCounter <= 0) {
                        _spinnerCounter = 0;
                        $("#globalSpinner").hide();
                    }
                };

    return {

        colours: {
            red: '#d1292',
            yellow: '#f8851b',
            green: '#99cb8e',
            blue: '#476e8c',
            black: '#000000',
            white: '#ffffff'
        },

        globalSpinner: {
                show:showSpinner,
                hide:hideSpinner
        },

        time: {
            // e.g. Jan 25th, 2008
            formatDate: function(javascriptTimestamp) {
                var date = new Date(javascriptTimestamp);
                return date.format('M dS Y');
            },

            // e.g. 22:42
            formatTimeOfDay: function(javascriptTimestamp) {
                var date = new Date(javascriptTimestamp);

                var hours = date.getHours() + "";
                var mins = date.getMinutes() + "";

                if (hours.length < 2) {
                    hours = "0" + hours;
                }

                if (mins.length < 2) {
                    mins = "0" + mins;
                }

                return hours + ":" + mins;
            },
            formatGMTOffset:function(javascriptTimestamp) {
                var date = new Date(javascriptTimestamp);
                return date.format('P');
            },

            format: function(javascriptTimestamp) {
                var convertDate = new Date(javascriptTimestamp);
                return  convertDate.format('Y-m-d H:i')+ ' (GMT ' + convertDate.format('P') + ')';
            },

            calculateNumberOfDays: function(milliseconds) {
                var oneDay = 24*60*60*1000; // hours*minutes*seconds*milliseconds
                return Math.round(Math.abs(milliseconds)/oneDay);
            }
        },

       /**
        *  Utitlity function to handle common ajax error senarios
        *
        *  $opts[statusCode] = ajax error code
        *  $opts[targetDiv] =  jquery object to show the error
        *  $opts[msg] =  optional message to show
        *  $opts[errorStatusMap] = object to override default msg and add new status and message
        *
        */
        showAjaxError: function($opts) {

            var statusCode = $opts.statusCode || null;
            var $errorDiv  = $opts.targetDiv || null;

            function showError($msg,$div) {
                var $errorDiv = $div || null;
                if ($errorDiv) {
                    var $span = $('<div style="margin-top:10px;">').addClass('alert alert-error').html($msg);
                    $errorDiv.empty().append($span).show();
                }
            }


            var errorStatusMap = {
                401: 'Access denied while trying to fetch data.',
                403: 'Unauthorized access.',
                404: 'Cannot fetch data as supplied url is not found.',
                500: 'Invalid server response when trying to fetch data.',
                0:   'Cannot contact the server.',
                204: 'No data available.'
            };

            $.extend(errorStatusMap, $opts.errorStatusMap);

            if (errorStatusMap[statusCode]) {
                showError(errorStatusMap[statusCode],$errorDiv);
            } else {
                showError($msg || "Cannot fetch data due to unknown error",$errorDiv);
            }
        },

        arrayWithValue: function(value, length) {
            var arr = new Array(length);
            for (var i = 0; i < arr.length; i++) {
                arr[i] = value;
            }
            return arr;
        },

        canonify: function(expression) {
            return expression.replace(/[^A-Za-z0-9]+/g, '_');
        },

        isValidClassExpression: function(expression) {
            try {
                new RegExp(expression);
                return true;
            } catch (e) {
                return false;
            }
        },

        randomUUID: function() {
            var S4 = function() {
                return (((1+Math.random())*0x10000)|0).toString(16).substring(1);
            };
            return (S4()+S4()+"-"+S4()+"-"+S4()+"-"+S4()+"-"+S4()+S4()+S4());
        },

        unixTimeToJavascriptTime: function(unixTime) {
            return unixTime * 1000;
        },

        javascriptTimeToUnixTime: function(javascriptTime) {
            return javascriptTime / 1000;
        },

        getLocalTimeStringFromUTC:function(utcTimeStamp){
            var d=new Date(0);
            d.setUTCSeconds(utcTimeStamp);

            var month=common.getDoubleDigits(d.getMonth()+1);
            var date=common.getDoubleDigits(d.getDate());
            var hour=common.getDoubleDigits(d.getHours());
            var minutes=common.getDoubleDigits(d.getMinutes());
            var dateString=d.getFullYear()+'-'+month+'-'+date+' '+hour+':'+minutes;
            return dateString;
        },

        getDoubleDigits:function(number){
            if(number < 10){
                return '0'+number;
            }
            return number;
        },

        generateRandomString: function (length) {
            let text = "";
            let dictionary = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*()}|?>:";

            for (let i = 0; i < length; i++)
            {
                text += dictionary.charAt(Math.floor(Math.random() * dictionary.length));
            }

            return text;
        }
    };
})();


var mediator = (function(){
    var subscribe = function(channel, fn){
        if (!mediator.channels[channel]) mediator.channels[channel] = [];
        mediator.channels[channel].push({
            context: this,
            callback: fn
        });
        return this;
    },

    publish = function(channel){
        if (!mediator.channels[channel]) return false;
        var args = Array.prototype.slice.call(arguments, 1);
        for (var i = 0, l = mediator.channels[channel].length; i < l; i++) {
            var subscription = mediator.channels[channel][i];
            subscription.callback.apply(subscription.context, args);
        }
        return this;
    };

    return {
        channels: {},
        publish: publish,
        subscribe: subscribe,
        installTo: function(obj){
            obj.subscribe = subscribe;
            obj.publish = publish;
        }
    };

}());

var stringToArray=function(commaSeparatedString){
    var resultfromExplode=commaSeparatedString.split(',');

    if(resultfromExplode.length==1 && resultfromExplode[0]=="" ){
        return [];
    }else{
        return  resultfromExplode;
    }
};

String.prototype.hashCode = function(){
    var hash = 0;
    if (this.length == 0) return hash;
    for (var i = 0; i < this.length; i++) {
        var charc = this.charCodeAt(i);
        hash = ((hash<<5)-hash)+charc;
        hash = hash & hash; // Convert to 32bit integer
    }
    return hash;
}

String.prototype.capitalizeFirstLetter = function(){
    return this.charAt(0).toUpperCase() + this.slice(1);
}

String.prototype.htmlSpecialChars =  function(){
    return this
        .replace(/&/g, '&amp;')
        .replace(/</g, '&lt;')
        .replace(/>/g, '&gt;')
        .replace(/"/g, '&quot;');
}

// Export node module.
if ( typeof module !== 'undefined' && module.hasOwnProperty('exports') )
{
    module.exports = common;
}

var unAuthorizedHandler = function () {
    alert('Your session has expired. You will be redirected to the Login page.');
    window.location.href = '/index.php/login/index';
}