var sketchController = BaseController.extend({
    /**
     * Initialize  Controller
     * @param $scope, current controller scope
     */

    // single sketch data
    sketch: null,
    _sketchService: null,
    _constants: null,
    sketchId: null,
    sketchFormInputModel: null,
    loadModalInstance: null,
    init: function($scope, $filter, $routeParams, $modal, $q, Notification, sketchService, DC_CONSTANTS, sketchFormInputModel, dashboardService, notificationService) {
        var self = this;
        this._notifications = Notification;
        this._sketchService = sketchService;
        this._dashboardService = dashboardService;
        this.sketchFormInputModel = sketchFormInputModel;
        this._constants = DC_CONSTANTS;
        this.$q = $q;
        this.$modal = $modal;
        $scope.$filter = $filter;
        this._super($scope);
        this.sketchId = $routeParams.sketchid;
        this.activationId = $routeParams.activationid;
        this.loadSketch(this.sketchId);

        this.ACTIVATION_NAME = '';

        this.$scope.goBack = function() {
            window.history.back();
        };


    },

    loadSketch: function(id) {
        var self = this;
        this._showLoadingDialog('Loading sketch');
        self.$scope.sketchView.name = 'deployed';
        var _sketchGetSuccess = function(result) {
                self.$scope.sketch = result.data;
                self._initializeActivationDetails(id);
                self.sketchFormInputModel.initializeInputTypes(result.data);
                self.$scope.sketchShow = true;
                self._hideLoadingDialog();
            };

        var _sketchGetError = function(error) {
                self._addProcessingAlerts('error', error.data);
            };

        var _gitResetSuccess = function() {
                self._sketchService.loadSketch(id).then(_sketchGetSuccess, _sketchGetError);
                self._hideLoadingDialog();
            };

        this._sketchService.resetGitWorkSpace().then(_gitResetSuccess, _sketchGetError);
    },


    _initializeActivationDetails: function(id) {
        var self = this;
        self.$scope.sketch.activations = [];
        self.$scope.isActivationViewLoading = true;
        self.$scope.activationLoadingError = "";
        var sketchDetailsLoaded = function(result) {
                activations = result.data;
                var length = activations.length;
                if (length === 0) {
                    self.$scope.sketchView.name = 'activate';
                    self.$scope.editMode = false;
                }
                for (var i = 0; i < length; i++) {

                    self.$scope.sketch.activations.push(activations[i]);
                    var activationIndex = activations[i].metadata.identifier;
                    self.$scope.activationDetails[activationIndex] = {};
                    var isOpen = (i === 0) ? true : false; // open 1st one by default
                    self.$scope.sketch.activations[i].isOpen = isOpen;

                    if (self.$scope.sketch.activations[i].identifier === self.activationId) {
                        self.$scope.sketch.activations[i].isOpen = true;
                    }
                }
                self.$scope.isActivationViewLoading = false;

            };
        var sketchDetailsLoadingFailed = function(error) {
                self.$scope.isActivationViewLoading = false;
                self.$scope.activationLoadingError = error.data;
            };

        this._sketchService.getActivationDetails(id).then(sketchDetailsLoaded, sketchDetailsLoadingFailed);

    },
    editActivation: function(activationDetails) {
        var self = this;
        // load all the inputs and bind model
        //
        self.sketchFormInputModel.initializeFormInputsFromActivationDetails(activationDetails);

        self.$scope.editingActivationId = activationDetails.metadata.identifier;
        self.$scope.environment.name = '(' + activationDetails.details.environments.activated.include.join(').(') + ')';
        self.$scope.environment.includes = activationDetails.details.environments.activated.include;
        self.$scope.editMode = true;
        self.$scope.sketchView.name = 'activate';

    },
    defineScope: function() {

        // this sets up the view scope for Form inputs
        this.sketchFormInputModel.setScope(this.$scope);

        this.$scope.sketchShow = false;
        this.$scope.hostClasses = {};
        this.$scope.showAlert = false;
        this.$scope.showLoading = false;
        this.$scope.sketchNotFound = false;
        this.$scope.editMode = false;
        this.$scope.alerts = [];


        this.$scope.environment = {};

        this.$scope.activationDetails = {};

        this.$scope.environment.name = '';


        this.$scope.sketchView = [];
        this.$scope.sketchView.name = "activate";

        this.$scope.showActivationView = this.showActivationView.bind(this);
        this.$scope.activate = this.activate.bind(this);
        this.$scope.closeAlert = this.closeAlert.bind(this);
        this.$scope.removeSketchActivation = this.removeSketchActivation.bind(this);
        this.$scope.editActivation = this.editActivation.bind(this);
        this.$scope.showPromiseNotKeptLog = this.showPromiseNotKeptLog.bind(this);
        this.$scope.hasDefaultValue = this.hasDefaultValue.bind(this);
        this.$scope.isObjectEmpty = this.isObjectEmpty.bind(this);


    },
    defineListeners: function() {

        var self = this;

        self.$scope.$on("showLoadingDialog", function(event, message) {
            self._showLoadingDialog(message);
        });

        self.$scope.$on("hideLoadingDialog", function() {
            self._hideLoadingDialog();
        });

        self.$scope.$on('commitChangesActivate_EVENT', function(event, data) {

            self._dashboardService.createAlert(data).then(

            function(result) {
                // do nothing
            }, function(error) {
                self.$scope.$emit('createAlertFail_Event', error.data);
            });
        });

        self.$scope.$on('commitChangesRemove_EVENT', function(event, data) {
                self._dashboardService.removeAlert(data).then(

                function(result) {
                    // do nothing
                }, function(error) {
                    self.$scope.$emit('removeAlertFail_Event', error.data);
                });
        });

        this._super();
    },

    hasDefaultValue: function(inputs) {

        return inputs.hasOwnProperty('default');

    },

    isObjectEmpty: function(obj) {
        if (obj === null) return true;
        return angular.equals({}, obj);
    },


    showActivationView: function(view) {
        var self = this;
        var show = view || 'activate';

        if (show == 'activate') {
            // this will pull and reset the user work repo
            // so that it will be working on latest repo
            this.$scope.editMode = false;
            this._showLoadingDialog('Configuring sketch for activation. Please wait...');
            this._sketchService.resetGitWorkSpace().then(

            function() {
                self._hideLoadingDialog();
                self.sketchFormInputModel.resetFormInputs();
                self.$scope.editingActivationId = '';
                self.$scope.environment.name = '';
                self.$scope.environment.includes = [];
                self.$scope.sketchView.name = show;

            },

            function(error) {
                self._addProcessingAlerts('error', error.data);
            });
        } else {
            self.$scope.sketchView.name = show;
        }
    },
    activate: function() {

        var self = this;

        var id = this.sketchId;

        this._showLoadingDialog('Activating sketch...');

        if (this.$scope.editMode === true) {
            // if edit mode then deactive 1st
            var data = self.sketchFormInputModel.getFormInputs();
            var activitationIdentifier = data.CF_MP_INTERNAL_SKETCH_activationName;
            this._sketchService.deactivateSketch(activitationIdentifier,this.$scope.sketch.metadata.name).then(

            function(result) {
                self._activateSketch(id);
            });
        } else {
            self._activateSketch(id);
        }
    },

    _activateSketch: function(sketchId) {
        var self = this;
        var data = self.sketchFormInputModel.getFormInputs();

        self.ACTIVATION_NAME = data.CF_MP_INTERNAL_SKETCH_activationName;

        var environment = this.$scope.environment;
        this._sketchService.activateSketch(sketchId, data, environment).then(

        function(result) {
            self._hideLoadingDialog();
            for (var sketchName in result.data) break;
            var  data = result.data[sketchName];
            data.sketchName = sketchName;
            data.includes = environment;
            self.openCommitDialog('activate', data);
        },

        function(error) {
            // activation error
            self._addProcessingAlerts('error', 'Something went wrong while activating:: ' + error.data);
        });
    },

    showPromiseNotKeptLog: function(hostkey, activationDetails) {

        var self = this;
        self.$scope.promiseNotKeptLogModel = {};


        var name = self.sketchId;
        this._sketchService.getPromiseNotKeptLog(hostkey, name, activationDetails.identifier).then(

        function(result) {
            self.$scope.promiseNotKeptLogModel.header = result.data.header;
            self.$scope.promiseNotKeptLogModel.rows = result.data.rows;
            self.$scope.promiseNotKeptLogModel.selectedHost = {
                'hostkey': hostkey
            };

            self.promiseLogModalInstance = self.$modal.open({
                templateUrl: 'promiseLogModal.html',
                controller: function($scope, $modalInstance, promiseNotKeptLogModel, ALLOW_CF_RUNAGENT) {

                    $scope.promiseNotKeptLogModel = promiseNotKeptLogModel;
                    $scope.ALLOW_CF_RUNAGENT = ALLOW_CF_RUNAGENT;

                    $scope.closePromiseNotKeptDialog = function() {
                        $modalInstance.close('cancel');
                    };


                },
                resolve: {
                    promiseNotKeptLogModel: function() {
                        return self.$scope.promiseNotKeptLogModel;
                    },
                    ALLOW_CF_RUNAGENT: function() {
                        return self.$scope.ALLOW_CF_RUNAGENT;
                    }
                }
            });


        },

        function(error) {});

    },


    removeSketchActivation: function(identifier, sketchName) {
        var self = this;
        self.ACTIVATION_NAME = identifier;
        this._showLoadingDialog('Removing sketch activation....');
        this._sketchService.deactivateSketch(identifier, sketchName).then(

        function(result) {
            self._addProcessingAlerts('success', 'deactivated');
            var data = {'sketchName':sketchName,'activationName':identifier};
            self.openCommitDialog('remove', data);
        },

        function(error) {
            // activation error
            self._addProcessingAlerts('error', 'Something went wrong while deactivating. ' + error.data);
        });

    },
    closeAlert: function(index) {
        this.$scope.alerts.splice(index, 1);
    },
    /**
     * @params {string} action - values: activate or remove
     */
    openCommitDialog: function(action, activationData) {

        var self = this;
        self.$scope.action = action || '';
        self.$scope.commitMessage = '';
        this._hideLoadingDialog();
        this.commitModalInstance = this.$modal.open({
            templateUrl: 'commitModal.html',
            backdrop: 'static',
            keyboard: false,
            controller: function($scope, $modalInstance, sketchService) {

                $scope.input = {};
                $scope.action = action;


                // check "create alert" checkbox
                if ($scope.action === 'activate') {
                    $scope.input.createAlert = true;
                }

                $scope.cancelCommit = function() {
                    sketchService.resetGitWorkSpace();
                    $modalInstance.close('cancel');
                };

                $scope.commitChanges = function() {
                    $modalInstance.close('cancel');
                    self._showLoadingDialog('Pushing changes to GIT...');
                    sketchService.commitChanges($scope.input.commitMessage).then(

                    function(result) {
                        var eventData = {};
                        eventData = activationData;

                        if ($scope.input.createAlert === true && self.$scope.action === 'activate') {
                            self.$scope.$broadcast('commitChangesActivate_EVENT', eventData);
                        } else if (self.$scope.action === 'remove') {
                            self.$scope.$broadcast('commitChangesRemove_EVENT', eventData);
                        }

                        self._hideLoadingDialog();
                        self.loadSketch(self.sketchId);
                    },

                    function(error) {
                        self._addProcessingAlerts('error', 'Error while pushing changes. ' + error.data);
                        sketchService.resetGitWorkSpace(); // reset the workspace
                    });
                };
            },
            resolve: {
                sketchService: function() {
                    return self._sketchService;
                }
            }
        });

        return this.commitModalInstance;


    },
    _showLoadingDialog: function(message) {
        var loadingMessage = message || null;
        var self = this;
        this.$scope.showLoading = true;
        this.$scope.loadingMessage = loadingMessage;
        this.$scope.processingAlert = [];
        this.loadModalInstance = this.$modal.open({
            templateUrl: 'loadingModal.html',
            controller: function($scope, $modalInstance, loadingMessage, processingAlert) {
                $scope.loadingMessage = loadingMessage;
                $scope.processingAlert = processingAlert;
                $scope.close = function() {
                    $modalInstance.close('cancel');
                };
            },
            resolve: {
                loadingMessage: function() {
                    return loadingMessage;
                },
                processingAlert: function() {
                    return self.$scope.processingAlert;
                }
            }
        });

        return this.loadModalInstance;


    },
    _hideLoadingDialog: function($modalInstance) {
        if (this.loadModalInstance) {
            this.loadModalInstance.close();
        }
    },
    _addProcessingAlerts: function(type, message) {
        var alert = {
            type: type,
            msg: message
        };

        this.$scope.processingAlert.push(alert);
    }
});

// we will have access to parent scope so all the sketch data is available here
var activationDetailLoadController = BaseController.extend({
    sketchService: null,
    filter: null,
    init: function($scope, sketchService, $filter, sketchFormInputModel) {
        this.sketchService = sketchService;
        this.sketchFormInputModel = sketchFormInputModel;
        this.filter = $filter;
        this._super($scope);
        this._setActivationStatus(this.$scope.activationViewModel);
    },
    defineScope: function() {
        this.$scope.inputTypesHasKey = this.inputTypesHasKey.bind(this);
        this.$scope.inputTypeGetDescriptionOrName = this.inputTypeGetDescriptionOrName.bind(this);
        this.$scope.getInputTypeForKey = this.getInputTypeForKey.bind(this);
    },

    _setActivationStatus: function(activationDetails) {

        if (activationDetails.details.hosts.length === 0) {
            this.$scope.activationStatus = 'warning';
        }

        var failedHost = this.filter('filter')(activationDetails.details.hosts, {
            status: 'failed'
        });


        if (failedHost.length > 0) {
            this.$scope.activationStatus = 'failed';
        } else {
            var waitingHost = this.filter('filter')(activationDetails.details.hosts, {
                status: 'warning'
            });
            if (waitingHost.length > 0) {
                this.$scope.activationStatus = 'warning';
            } else {
                this.$scope.activationStatus = 'success';
            }
        }
    },

    inputTypesHasKey: function(key, activationDetails) {


        // get the bundle
        //
        var bundle = activationDetails.bundle;
        var inputTypes = this.sketchFormInputModel.getInputTypesOfBundle(bundle);

        var hasKey = this.filter('filter')(inputTypes, {
            'name': key
        }).length;
        return hasKey;
    },

    getInputTypeForKey: function(key, activationDetails) {

        var bundle = activationDetails.bundle;

        var inputTypes = this.sketchFormInputModel.getInputTypesOfBundle(bundle);

        var element = this.filter('filter')(inputTypes, {
            'name': key
        }, true);


        if (element.length) return element[0].type;
    },

    inputTypeGetDescriptionOrName: function(key, activationDetails) {

        var bundle = activationDetails.bundle;

        var inputTypes = this.sketchFormInputModel.getInputTypesOfBundle(bundle);

        var element = this.filter('filter')(inputTypes, {
            'name': key
        }, true);

        var returnValue = '';
        if (element && element.length) {
            returnValue = element[0].description ? element[0].description : element[0].name;
        }
        return returnValue;

    }

});


sketchController.$inject = ['$scope', '$filter', '$routeParams', '$modal', '$q', 'Notifications', 'sketchService', 'DC_CONSTANTS', 'sketchFormInputModel', 'dashboardService', 'notificationService'];
activationDetailLoadController.$inject = ['$scope', 'sketchService', '$filter', 'sketchFormInputModel'];