/**
 * Set Angular directive base on the class above,
 * this gives us a lot more flexibility and permits
 * inheritance over Directives
 */
angular.module('dc.directives', []).directive('category', ['sketchService', '$filter', function(sketchService, filter) {

    return {
        restrict: 'A',
        isolate: true,
        templateUrl: "designcenter/partials/sketchCategory",
        link: function($scope, $elm, $attrs) {
            new SketchCategoryDirective($scope, $elm, $attrs, sketchService, filter);
        },
        scope: true
    };
}]).directive('environment', ['naviTreeService', '$filter','$modal', function(naviTreeService, $filter,$modal) {
    return {
        restrict: 'A',
        link: function($scope, $elm, $attrs) {
            new environmentSelectionDirective($scope, $elm, $attrs, naviTreeService, $filter,$modal);
        },
        scope: {
            env: '=',
            showHosts: '=',
            sketch: '='
        },
        templateUrl: 'designcenter/partials/sketchEnvironmentSelection'
    };
}]).directive('jsonInputType', ['sketchService', '$filter', '$q', '$timeout', function(sketchService, $filter, $q, $timeout) {
    return {
        restrict: 'A',
        require: '^form',
        link: function($scope, $elm, $attrs, formController) {
            new jsonInputType($scope, $elm, $attrs, formController, sketchService, $filter, $q, $timeout);
        },
        scope: {
            inputs: "=metaData",
            forms: "=source"
        },
        templateUrl: 'designcenter/partials/sketchJsonInput'
    };
}]).directive('arrayInputType', ['sketchService', '$filter', '$q', function(sketchService, $filter, $q) {
    return {
        restrict: 'A',
        require: '^form',
        link: function($scope, $elm, $attrs, formController) {
            new arrayInputType($scope, $elm, $attrs, formController, sketchService, $filter, $q);
        },
        scope: {
            inputs: "=metaData",
            forms: "=source"
        },
        templateUrl: 'designcenter/partials/sketchArrayInput'
    };
}])

.directive('dcvalidation', ['sketchService', function(sketchService) {
    return {
        restrict: 'A',
        require: '?ngModel',
        link: function($scope, $elm, $attrs, ngModel) {
            new dcValidation($scope, $elm, $attrs, ngModel, sketchService);
        }
    };
}]).directive('validateDescription', ['sketchService', function(sketchService) {
    return {
        restrict: 'A',
        controller: function($scope) {},
        link: function($scope, $elm, $attrs, ctrl) {
            new descriptionValidation($scope, $elm, $attrs, ctrl, sketchService);
        },
        require: 'ngModel',
        scope: {
            alist: '=',
            editid: '='
        }
    };
}]).directive('dcvalidationArray', ['sketchService', function() {
    return {
        restrict: 'A',
        link: function($scope, $elm, $attrs, ctrl) {
            new dcArrayRequiredValidation($scope, $elm, $attrs, ctrl);
        },
        require: '^ngModel'
    };
}]).directive('csvList', function() {
    return {
        link: function($scope, $elm, $attrs, ctrl) {
            new csvListDirective($scope, $elm, $attrs, ctrl);
        },
        require: 'ngModel'
    };
}).directive('notifications', ['notificationService', '$timeout', function (notificationService, $timeout) {
    return {
        restrict: 'A',

        template: '<div id="notification_block" ng-repeat="(type, notification) in notificationsList">'+
                    '<div ng-show="type==\'error\'"   class="alert alert-error"><button class="close" data-dismiss="alert">×</button>{{notification.text}}</div>' +
                    '<div ng-show="type==\'success\'" class="alert alert-info"><button class="close" data-dismiss="alert">×</button>{{notification.text}}</div>' +
                    '<div ng-show="type==\'warning\'" class="alert alert-warning"><button class="close" data-dismiss="alert">×</button>{{notification.text}}</div>' +
                '</div>',

        link: function($scope, $elm, $attrs) {
            $scope.$on('setNotification', function () {
                $scope.notificationsList = notificationService.getNotifications();

                if ($scope.modalOpen === undefined || $scope.modalOpen.status === false) {
                    window.scrollTo(null, (angular.element("#notification_block_wrapper").position().top - 40));
                    if ($(".alert-info")) {
                        // make sure it displays again for multiple actions after initial fadeTo
                       $(".alert-info").fadeTo(0,1);
                    }
                    $timeout(function() {
                        if ($(".alert-info")) {
                            //check for alert-info, fade and hide if exists
                            $(".alert-info").fadeTo(500, 0).slideUp(500);
                        }
                    }, 5000);
                }
            });

            $scope.$on('resetNotificationMessage', function () {
                $scope.notificationsList = {};
            });


            $scope.$on('modalOpen', function () {
                $scope.modalOpen = {'status':true};
            });


            $scope.$on('modalClose', function () {
                $scope.modalOpen = {'status':false};
            });
        }
    };
}]).directive('chozen', function(){
    var linker = function(scope,element,attr) {
        element.chosen({disable_search_threshold:5});
        scope.$watch(attr.ngModel, function(){
            element.trigger('chosen:updated');
        })
    };
    return {
        restrict:'A',
        link: linker
    };
});