var sketchFormInputModel = Class.extend({
    //Injected by the provider
    inputTypes: [],
    formsInput: {},
    $q: null,
    count: 0,
    scope: null,
    inputTypeEntryPointHash: {},

    /**
     * initializes input type of sketches
     * @param  {object} $sketch [whole sketch object]
     * @return void         [description]
     */
    initializeInputTypes: function(sketch) {
        this.scope.entryPointLists = [];
        for (var p in sketch.api) {
            var entryPointData  = {};
            entryPointData.key = p;
            var meta = this.scope.$filter('filter')(sketch.api[p], {
                                     type: 'bundle_options'
                                    });
            entryPointData.meta = meta[0] || {};
            this.scope.entryPointLists.push(entryPointData);
            this.scope.entryPoint.value = p;
            var inputs = sketch.api[p];
            this.inputTypes = inputs;
            this.inputTypeEntryPointHash[p] = inputs;
        }
        this.scope.inputTypes = this.inputTypes;
        this.formsInput['CF_MP_INTERNAL_SKETCH_activationName'] = '';
        this.resetFormInputs();

    },

    changeEntryPoint: function(newEntryPoint) {
        this.scope.entryPoint = newEntryPoint;
        var inputs = this.inputTypeEntryPointHash[newEntryPoint.value];
        this.inputTypes = inputs;
        this.scope.inputTypes = this.inputTypes;
        this.resetFormInputs();

    },

    initializeFormInputsFromActivationDetails: function(activationDetails) {

        this.scope.entryPoint.value = activationDetails.bundle;
        if (!this.scope.entryPoint.value) {
            // if no bundle is defined take the first one from the list
            for (var k in this.inputTypeEntryPointHash) {
                this.scope.entryPoint.value = k;
                break;
            }

        }


        var inputs = this.inputTypeEntryPointHash[this.scope.entryPoint.value];
        this.inputTypes = inputs;
        this.scope.inputTypes = this.inputTypes;
        this.resetFormInputs();
        var params = activationDetails.details.params;
        for (var key in params) {
            this.formsInput[key] = params[key];
        }
        this.formsInput['CF_MP_INTERNAL_SKETCH_activationName'] = activationDetails.metadata.identifier;
        this.scope.formsInput = this.formsInput;
    },

    resetFormInputs: function() {
        var self = this;
        var params = this.inputTypes;
        var tempActivationName = this.formsInput['CF_MP_INTERNAL_SKETCH_activationName'];
        this.formsInput = {};
        for (var key in params) {
            var metadata = params[key];
            if (metadata.name) {
                if (metadata.type == 'list') {
                    this.formsInput[metadata.name] = [""]; // add empty default for lists
                } else if (metadata.type == 'array') {
                    var defaults = metadata['default'] ? metadata['default'] : null;
                    this.formsInput[metadata.name] = defaults; // add empty default for array
                }
            }
        }

        this.formsInput['CF_MP_INTERNAL_SKETCH_activationName'] = tempActivationName;
        this.formsInput['CF_MP_ENTRY_POINT'] = this.scope.entryPoint.value; // always pass the entry point
        this.scope.formsInput = this.formsInput;
    },
    /**
     * Actual input types of the sketches
     * @return {object}
     */
    getInputTypes: function() {
        return this.scope.inputTypes;
    },

    /**
     * Used for form inputs in sketches
     * @return {object}
     */
    getFormInputs: function() {
        return this.scope.formsInput;
    },

    getInputTypesOfBundle: function(bundle) {
        if (!bundle) {
            // if empty bundle is supplied return the first one
            for (var k in this.inputTypeEntryPointHash) {
                bundle = k;
                break;
            }
        }
        return this.inputTypeEntryPointHash[bundle];
    },

    hasKeyValueValidation: function(inputs) {
        var returnValue = false;
        if (this.scope.sketch.validationDetails && inputs.validation) {
            var validationDetail = this.scope.sketch.validationDetails[inputs.validation];
            returnValue = validationDetail && validationDetail.hasOwnProperty('array_k') && validationDetail.hasOwnProperty('array_v');
        }
        return returnValue;
    },

    setScope: function(scope) {
        this.scope = scope;
        this.scope.entryPointLists = [];
        this.scope.entryPoint = {};
        this.scope.formsInput = {};
        this.scope.inputTypes = {};
        this.scope.dcServerValidationMessage = {};
        this.scope.validationDetails = {};

        this.scope.hasKeyValueValidation = this.hasKeyValueValidation.bind(this);
        this.scope.changeEntryPoint = this.changeEntryPoint.bind(this);
    }


});